
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

import sys
import time

from AccessMath.data.meta_data_DB import MetaDataDB
from AccessMath.preprocessing.config.parameters import Parameters
from AccessMath.preprocessing.content.keyframe_exporter import KeyframeExporter
from AccessMath.util.time_helper import TimeHelper

def main():
    if len(sys.argv) < 4:
        print("Usage:")
        print("")
        print(" python export_binary_video.py database lecture mode")
        print("")
        print("Where")
        print("\tDatabase:\tPath to AccessMath database file (in XML format)")
        print("\tlecture:\tLecture to export")
        print("\tmode:\t\tVideo Mode")
        print("\t\t0 - Raw Binary")
        print("\t\t1 - Raw Binary (Inverted)")
        print("\t\t2 - Binary Background Removed")
        print("\t\t3 - Binary Background Removed (Inverted)")
        print("\t\t4 - Binary Reconstructed")
        print("\t\t5 - Binary Reconstructed (Inverted)")
        return

    skip_interpolation = 2 # 15
    start_time = time.time()

    try:
        video_mode = int(sys.argv[3])
        if video_mode < 0 or video_mode >= 6:
            print("Invalid Video Mode")
            return
    except:
        print("Invalid Video Mode")
        return

    database, lecture = MetaDataDB.load_database_lecture(sys.argv[1], sys.argv[2])
    if database is None:
        return

    if video_mode == 0 or video_mode == 1:
        source = Parameters.Output_Binarize
        output_prefix = "binary"
        invert_binary = video_mode == 1
    elif video_mode == 2 or video_mode == 3:
        source = Parameters.Output_BG_Removal
        output_prefix = "bgmrem"
        invert_binary = video_mode == 3
    elif video_mode == 4 or video_mode == 5:
        source = Parameters.Output_CCReconstructed
        output_prefix = "reconstructed"
        invert_binary = video_mode == 5
    else:
        print("Invalid Video Mode")
        return

    KeyframeExporter.ExportVideo(database, lecture, source, output_prefix, invert_binary, skip_interpolation)

    end_time = time.time()

    print("\nTotal Elapsed Time: " + TimeHelper.secondsToStr(end_time - start_time))
    print("Finished")


if __name__ == '__main__':
    main()