
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

import sys
import pygame
import traceback
from AccessMath.data.meta_data_DB import MetaDataDB

from AccessMath.annotation.gt_keyframe_annotator import GTKeyFrameAnnotator

def main():
    if len(sys.argv) < 3:
        print("Usage: python gt_keyframes.py database lecture")
        print("Where")
        print("\tdatabase\t= Database metadata file")
        print("\tlecture\t\t= Lecture video to process")
        print("")
        return

    small_mode = (len(sys.argv) >= 4) and (sys.argv[3].lower() == "small")
    large_mode = (len(sys.argv) >= 4) and (sys.argv[3].lower() == "large")

    database, current_lecture = MetaDataDB.load_database_lecture(sys.argv[1], sys.argv[2])
    if database is None:
        return

    # input/output files path
    output_path = database.output_annotations + "/" + database.name + "_" + current_lecture.title.lower()

    pygame.init()
    pygame.display.set_caption('Access Math - Ground Truth Annotation Tool - ' + database.name + "/" + current_lecture.title)

    if small_mode:
        screen_w, screen_h = 1250, 670
    else:
        if large_mode:
            screen_w, screen_h = 1850, 970
        else:
            # default res....
            screen_w, screen_h = 1500, 820

    window = pygame.display.set_mode((screen_w, screen_h))
    background = pygame.Surface(window.get_size())
    background = background.convert()

    main_menu = GTKeyFrameAnnotator(window.get_size(), database.name, current_lecture.title, output_path)

    """
    print("restore the main file")
    for i in range(3):
        main_menu.btn_nav_keyframe_next_click(None)
    main_menu.btn_portions_show_click(None)
    for i in range(20):
        main_menu.btn_portions_next_click(None)
    main_menu.btn_portions_edit_click(None)
    main_menu = main_menu.return_screen
    """

    current_screen = main_menu
    current_screen.prepare_screen()
    prev_screen = None

    while not current_screen is None:
        #detect when the screen changes...
        if current_screen != prev_screen:
            #remember last screen...
            prev_screen = current_screen

        #capture events...
        current_events = pygame.event.get()
        try:
            current_screen = current_screen.handle_events(current_events)
        except Exception as e:
            print("An exception ocurred")
            print(e)
            traceback.print_exc()

        if current_screen != prev_screen:
            if current_screen != None:
                #prepare the screen for new display ...
                current_screen.prepare_screen()

        #draw....
        background.fill((0, 0, 0))

        if not current_screen is None:
            current_screen.render(background)

        window.blit(background, (0, 0))
        pygame.display.flip()

if __name__ == "__main__":
    main()
