
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

#============================================================================
# Preprocessing Model for ST3D indexing - V 1.1
#
# Kenny Davila
# - Created:  September 8, 2015
#
#============================================================================

import sys
import math
import numpy as np
import matplotlib.pyplot as plt

from AccessMath.preprocessing.config.parameters import Parameters
from AccessMath.preprocessing.content.helper import Helper
from AccessMath.preprocessing.content.video_segmenter import VideoSegmenter
from AccessMath.preprocessing.user_interface.console_ui_process import ConsoleUIProcess

#============================================================================
# based on the method in paper:
#     CONTENT EXTRACTION AND SUMMARIZATION OF INSTRUCTIONAL VIDEOS
#     by Tiecheng Liu and Chekuri Choudary
#============================================================================
def select_keyframes_using_window(sums, k):
    i = 0
    max_value = 0
    keyframe_indices = []
    while i < len(sums):
        # find the index of the maximum value for sums in the current window
        t = i
        for t2 in range(i + 1, min(i + k, len(sums))):
            if sums[t2] > sums[t]:
                t = t2

        if sums[t] <= max_value:
            keyframe_indices.append(t)
            max_value = 0
            i = i + k
        else:
            max_value = sums[t]
            i = t

    return keyframe_indices

def process_input(process, input_data):
    frame_times, frame_indices, compressed_frames = input_data

    # 1) decompress all images ...
    print("Decompressing input...")
    all_binary = Helper.decompress_binary_images(compressed_frames)

    # 2) Computing all sums...
    print("Computing sums...")
    all_sums = VideoSegmenter.compute_binary_sums(all_binary)

    # selecting k to be close to the original in the paper (25 seconds)
    k = int(round(Parameters.Sampling_FPS * 25))
    keyframe_indices = select_keyframes_using_window(all_sums, k)

    kf_frame_times = []
    kf_frame_indices = []
    kf_compressed_frames = []

    for kf_idx in keyframe_indices:
        kf_frame_times.append(frame_times[kf_idx])
        kf_frame_indices.append(frame_indices[kf_idx])
        kf_compressed_frames.append(compressed_frames[kf_idx])

    return kf_frame_times, kf_frame_indices, kf_compressed_frames

def main():
    #usage check
    if not ConsoleUIProcess.usage_check(sys.argv):
        return

    # using SUMS
    inputs = Parameters.Output_CCReconstructed

    process = ConsoleUIProcess(sys.argv[1], sys.argv[2:], inputs, Parameters.Output_SUM_Vid_Segment)
    if not process.initialize():
       return

    process.start_input_processing(process_input)

    print("Finished")

if __name__ == "__main__":
    main()