
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

#============================================================================
# Preprocessing Model for ST3D indexing - V 1.1
#
# Kenny Davila
# - Created:  March 7, 2017
#
#============================================================================

import sys

from AccessMath.preprocessing.config.parameters import Parameters
from AccessMath.preprocessing.video_worker.ROI_detection import ROIDetection
from AccessMath.preprocessing.content.hough_roi_detector import HoughROIDetector
from AccessMath.preprocessing.content.MLBinarizer import MLBinarizer
from AccessMath.preprocessing.user_interface.console_ui_process import ConsoleUIProcess

def get_worker(process):
    # set up region of interest detector ....
    roi_detector = HoughROIDetector(Parameters.ROIDet_Hough_rho, Parameters.ROIDet_Hough_theta,
                                    Parameters.ROIDet_Hough_min_intersections, Parameters.ROIDet_Hough_min_line_length,
                                    Parameters.ROIDet_Hough_max_line_gap, Parameters.ROIDet_Hough_diag_threshold,
                                    Parameters.ROIDet_workers)

    ml_binarizer = MLBinarizer(None, None, Parameters.MLBin_sigma_color, Parameters.MLBin_sigma_space,
                               Parameters.MLBin_median_blur_k, Parameters.MLBin_dark_background)

    # create video worker
    worker = ROIDetection(ml_binarizer, Parameters.ROIDet_temporal_blur_K, Parameters.ROIDet_bin_threshold,
                          Parameters.ROIDet_edge_min_threshold, Parameters.ROIDet_edge_max_threshold, roi_detector)

    # enable image output ...
    worker.set_debug_mode(True, 0, 0, "", process.current_lecture.title)
    #print("\nREMOVE THE NO ROI DEBUG MODE!\n")
    #worker.debug_no_ROI = True

    return worker

def get_results(worker):
    return worker.ROI_edges, worker.ROI_polygon_mask, worker.ROI_score

def main():
    #usage check
    if not ConsoleUIProcess.usage_check(sys.argv):
        return

    process = ConsoleUIProcess(sys.argv[1], sys.argv[2:], None, Parameters.Output_ROI_Detection)

    if not process.initialize():
        return

    fps = Parameters.ROIDet_fps
    process.start_video_processing(fps, get_worker, get_results, 0, True)

    print("finished")

if __name__ == "__main__":
    main()