
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

#============================================================================
# Preprocessing Model for ST3D indexing - V 1.1
#
# Kenny Davila
# - Created:  March 10, 2017
#
#============================================================================

import sys
import pickle

from AccessMath.preprocessing.config.parameters import Parameters
from AccessMath.preprocessing.content.MLBinarizer import MLBinarizer
from AccessMath.preprocessing.video_worker.ML_frame_binarizer import MLFrameBinarizer
from AccessMath.preprocessing.user_interface.console_ui_process import ConsoleUIProcess

def get_worker(process):
    # load classifier from file ...
    in_file = open(Parameters.MLBin_classifier_file, "rb")
    classifier = pickle.load(in_file)
    in_file.close()

    ml_binarizer = MLBinarizer(classifier, Parameters.MLBin_patch_size, Parameters.MLBin_sigma_color,
                               Parameters.MLBin_sigma_space, Parameters.MLBin_median_blur_k,
                               Parameters.MLBin_dark_background)

    worker_binarizer = MLFrameBinarizer(ml_binarizer, Parameters.MLBin_hysteresis)

    # worker_binarizer.set_debug_mode(True, 0, 5000000, "output/images", process.current_lecture.title)
    worker_binarizer.set_debug_mode(True, 0, 50000, "output/images", process.current_lecture.title)
    return worker_binarizer


def get_results(worker):
    return (worker.frame_times, worker.frame_indices, worker.compressed_frames)


def main():
    #usage check
    if not ConsoleUIProcess.usage_check(sys.argv):
        return

    process = ConsoleUIProcess(sys.argv[1], sys.argv[2:], None, Parameters.Output_Binarize)
    if not process.initialize():
        return

    fps = Parameters.Sampling_FPS
    process.start_video_processing(fps, get_worker, get_results, 0, True)

    print("finished")

if __name__ == "__main__":
    main()
