
# ==============================================================================
#	AccessMath V 0.1 - Lecture Video Summarization
#	Copyright (C) 2017  Kenny Davila, Richard Zanibbi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ===============================================================================

#============================================================================
# Preprocessing Model for ST3D indexing - V 1.1
#
# Kenny Davila
# - Created:  March 10, 2017
#
#============================================================================

import sys

from AccessMath.preprocessing.config.parameters import Parameters
from AccessMath.preprocessing.content.helper import Helper
from AccessMath.preprocessing.content.roi_background_remover import ROIBackgroundRemover
from AccessMath.preprocessing.user_interface.console_ui_process import ConsoleUIProcess


def process_input(process, input_data):
    frame_times, frame_indices, compressed_frames = input_data[0]
    ROI_edges, ROI_polygon_mask, ROI_score = input_data[1]

    # 1) decompress all images ...
    print("Decompressing input...")
    all_binary = Helper.decompress_binary_images(compressed_frames)

    # 2) Subtract background from each binary image
    #save_prefix = process.img_dir  + "/bg_bin_" + process.current_lecture.title
    save_prefix = None
    print("Removing background...")
    min_overlap = Parameters.ROIBGRem_min_overlap
    compressed_output = ROIBackgroundRemover.remove_background(all_binary, ROI_polygon_mask, min_overlap, True,
                                                               save_prefix)

    return frame_times, frame_indices, compressed_output


def main():
    #usage check
    if not ConsoleUIProcess.usage_check(sys.argv):
        return

    process = ConsoleUIProcess(sys.argv[1], sys.argv[2:], [Parameters.Output_Binarize, Parameters.Output_ROI_Detection],
                               Parameters.Output_BG_Removal)

    if not process.initialize():
       return

    process.start_input_processing(process_input)

    print("Finished")

if __name__ == "__main__":
    main()